#!/usr/bin/python3
#/* vim:set ts=2 set noexpandtab */
import json, uuid, hashlib, sqlite3, base64
from hashlib import sha1
import bencoder
import requests
from flask import Flask, render_template, url_for, request, send_file
from werkzeug import secure_filename
app = Flask(__name__)
strings = None
settings = None

@app.route("/")
def index():
	return render_template("search.html", language="english", categories=settings["categories"], strings=strings)

@app.route("/categories")
def categorys():
	global strings
	return render_template("categories.html", categories=settings["categories"], strings=strings)

@app.route("/create", methods=['GET','POST'])
def create():
	if request.method == "GET":
		return render_template("create.html", language="english", categories=settings["categories"], strings=strings, errors=None)
	elif request.method == "POST":
		errors = createNewTorrent(request)
		if errors == None:
			return "It's allright"
		else:
			return render_template("create.html", language="english", categories=settings["categories"], strings=strings, errors=errors)

@app.route("/download/<filename>")
def download(filename):
	connection = sqlite3.connect("torrentdb.sqlite")
	c = connection.cursor()
	c.execute("SELECT name FROM torrents WHERE fileid = :fileid", { 'fileid' : filename})
	name = c.fetchone()[0]
	return send_file("torrentFiles/" + filename, as_attachment=True, attachment_filename=name + ".torrent", conditional=True)

@app.route("/search", methods=['GET'])
def search():
	global strings
	print(strings)
	connection = sqlite3.connect("torrentdb.sqlite")
	c = connection.cursor()

	search_params = []
	search = ""
	fields = list(request.args.keys())
	for field in fields:
		query_list = request.args.getlist(field)
		for query in query_list:
			if len(search) > 0:
				search += " AND "

			if field is "q":
				names = query.split(" ")
				search_params += list(map(lambda x: "%" + x + "%", names))
				search += " AND ".join(["torrents.name LIKE (?)"] * len(query.split(" ")))
			elif field is "c":
				search_params += query.split(" ")
				search += " AND ".join(["torrents.category LIKE (?)"] * len(query.split(" ")))
			elif field is "s":
				search_params += query.split(" ")
				search += " AND ".join(["torrents.subcategory LIKE (?)"] * len(query.split(" ")))
			elif field is "h":
				hashes = query.split(" ")
				search_params += list(map(lambda x: x + "%", hashes))
				search += " AND ".join(["torrents.fileid LIKE (?)"] * len(query.split(" ")))

	print(search)
	c.execute("SELECT torrents.fileid, torrents.name, metadata.torrentsize FROM torrents LEFT JOIN metadata on metadata.fileid = torrents.fileid WHERE " + search, search_params)
	results = c.fetchall()
	return render_template("result.html", results=results, strings=strings, language="english", categories=settings["categories"])

def scrapeAll():
	TRACKER_URL = "http://tracker.lootbox.cf:6969/"
	statedump = requests.get(TRACKER_URL + "stats" + "?mode=statedump")

	return

def init():
	global strings
	global settings
	with open("strings.json") as stringsJson:
		strings = json.load(stringsJson)

	with open("settings.json") as settingsJson:
		settings = json.load(settingsJson)
	initDb()
	#scrapeAll()

def initDb():
	connection = sqlite3.connect("torrentdb.sqlite")
	c = connection.cursor()
	c.execute('CREATE TABLE IF NOT EXISTS torrents (fileid TEXT PRIMARY KEY NOT NULL, name TEXT NOT NULL, category TEXT NOT NULL, subcategory TEXT NOT NULL, description TEXT NOT NULL, audioquality_description TEXT NOT NULL, videoquality_description TEXT NOT NULL);')
	c.execute('CREATE TABLE IF NOT EXISTS metadata (fileid TEXT PRIMARY KEY NOT NULL, created_by TEXT, creation_date TEXT, announce_url TEXT NOT NULL, source TEXT, torrentsize TEXT NOT NULL, name TEXT NOT NULL, private TEXT NOT NULL)')
	connection.commit()
	connection.close()

def getLocalString(language, descriptor):
	global strings
	if language in strings.keys():
		if descriptor in strings[language].keys():
			return strings[language][descriptor]
		else:
			return descriptor
	else:
		return descriptor

def createNewTorrent(reuqest):
	uploadfile = request.files["torrentFile"]
	filename = secure_filename(uploadfile.filename)

	content = request.files["torrentFile"].stream.read()
	bcoded = bencoder.decode(content)
	info_hash = sha1(bencoder.encode(bcoded[b'info'])).hexdigest()

	with open("torrentFiles/" + info_hash, "wb") as torrent_file:
		torrent_file.write(content)
	
	bcoded = bencoder.decode(content)
	size = ((len(bcoded[b'info'][b'pieces']) / 20) * bcoded[b'info'][b'piece length']) / 1024 / 1024

	print("=== CREATE NEW TORRENT FILE ===")
	print( "Name:         " + request.form["name"] )
	print( "Torrent file: " + info_hash )
	print( "Category:     " + request.form["category"] )
	print( "Subcategory:  " + request.form["subcategory"] )
	print( "Description:  " + request.form["description"] )

	#TODO: Validate the input serverside before writing it to the database
	name = request.form["name"]
	category = request.form["category"]
	subcategory = request.form["subcategory"]
	description = request.form["description"]
	audioquality_description = request.form["audioquality_description"]
	videoquality_description = request.form["videoquality_description"]
	newTFile = TorrentFile(info_hash, name, category, subcategory, description, audioquality_description, videoquality_description)
	try:
		connection = sqlite3.connect("torrentdb.sqlite")
		newTFile.writeToDb(connection.cursor())
		newTFile.metadata.writeToDb(connection.cursor())
		connection.commit()
		connection.close()
		return ["Success"]
	except sqlite3.IntegrityError as e:
		print(e)
		return ["Torrent <a href=\"/search?h={}\">{}</a> does already exist".format(info_hash, info_hash[:-20])]
	except Exception as e:
		print(e)
		return ["Unknown error in creation"]

class Metadata():
	def __init__(self, fileid):
		with open("torrentFiles/" + fileid, "rb") as f:
			torrent = f.read()
		self.fileid = fileid
		self.bcoded = bencoder.decode(torrent)
		self.created_by = self.bcoded.get(b'created by', "")
		self.creation_date = self.bcoded.get(b'creation date', "")
		self.announce_url = self.bcoded.get(b'info', dict()).get(b'', "")
		self.source = self.bcoded.get(b'info', dict()).get(b'source', "")
		self.torrentsize = ((len(self.bcoded.get(b'info', dict()).get(b'pieces', "")) / 20) * self.bcoded.get(b'info', dict()).get(b'piece length'))
		self.name = self.bcoded.get(b'info', dict()).get(b'name', "")
		self.private = self.bcoded.get(b'info', dict()).get(b'private', "")

	def writeToDb(self, cursor):
		c = cursor
		b64created_by = base64.b64encode(self.created_by)
		b64announce_url = base64.b64encode(self.announce_url.decode()) if self.announce_url else ""
		b64source = base64.b64encode(self.source) if self.source else ""
		b64name = base64.b64encode(self.name)
		c.execute("INSERT INTO metadata(fileid, created_by, creation_date, announce_url, source, torrentsize, name, private) VALUES(:fileid, :created_by, :creation_date, :announce_url, :source, :torrentsize, :name, :private)", { 'fileid' : self.fileid, 'created_by' : b64created_by, 'creation_date' : self.creation_date, 'announce_url' : b64announce_url, 'source' : b64source , 'torrentsize' : self.torrentsize, 'name' : b64name, 'private' : self.private})

class TorrentFile():
	fileid = None
	name = None
	category = None
	subcategory = None
	description = None
	audioquality_description = None
	videoquality_description = None
	def __init__(self, fileid=fileid, name=name, category=category, subcategory=subcategory, description=description, audioquality_description=audioquality_description, videoquality_description=videoquality_description):
		self.fileid = fileid
		self.name = name
		self.category = category
		self.subcategory = subcategory
		self.description = description
		self.audioquality_description = audioquality_description
		self.videoquality_description = videoquality_description
		self.metadata = Metadata(fileid)

	def writeToDb(self, cursor):
		c = cursor
		b64description = base64.b64encode(self.description.encode())
		b64audioquality_description = base64.b64encode(self.audioquality_description.encode())
		b64videoquality_description = base64.b64encode(self.videoquality_description.encode())
		c.execute("INSERT INTO torrents(fileid, name, category, subcategory, description, audioquality_description, videoquality_description) VALUES(:fileid, :name, :category, :subcategory, :description, :audioquality_description, :videoquality_description)", { 'fileid' : self.fileid, 'name' : self.name, 'category' : self.category, 'subcategory' : self.subcategory, 'description' : b64description , 'audioquality_description' : b64audioquality_description, 'videoquality_description' : b64videoquality_description})

if __name__ == "__main__":
	init()
	app.jinja_env.globals.update(getLocalString=getLocalString)
	app.jinja_env.globals.update(json=json)
	app.jinja_env.globals.update(sorted=sorted)
	app.run(debug=True, host="127.0.0.1")
